from __future__ import absolute_import, division, print_function, unicode_literals
from sdss_access.utils.six.moves import input

""" add the following username and password to your ~/.netrc file
    and remember to chmod 600 ~/.netrc

machine data.sdss.org
    login sdss
    password ***-******
"""

class Auth:

    def __init__(self, netloc=None, public=False, verbose=False):
        self.public = public
        self.verbose = verbose
        self.set_netloc(netloc=netloc)
        self.set_netrc()
        
    def set_netrc(self):
        try:
            from netrc import netrc
            self.netrc = netrc() if not self.public else None
        except Exception as e:
            if self.verbose: print("SDSS_ACCESS> AUTH NETRC: %r" % e)
            self.netrc = None

    def set_netloc(self, netloc=None):
        self.netloc = netloc

    def set_username(self, username=None, inquire=False):
        self.username = input("user [sdss]: ") or "sdss" if inquire else username

    def set_password(self, password=None, inquire=False):
        try:
            from getpass import getpass
            self.password = getpass("password: ") if inquire else password
        except Exception as e:
            if self.verbose: print("SDSS_ACCESS> AUTH PASSWORD: %r" % e)
            self.password = None

    def ready(self):
        return self.username and self.password

    def load(self):
        if self.netloc and self.netrc:
            authenticators = self.netrc.authenticators(self.netloc)
            if authenticators and len(authenticators)==3:
                self.set_username(authenticators[0])
                self.set_password(authenticators[2])
                if self.verbose: print("authentication for netloc=%r set for username=%r " % (self.netloc,self.username))
            else:
                if self.verbose: print("cannot find %r in ~/.netrc" % self.netloc)
                self.set_username()
                self.set_password()
        else:
            self.set_username()
            self.set_password()



#!/usr/bin/env python
# encoding: utf-8
#
# spaxel.py
#
# Licensed under a 3-clause BSD license.
#
# Revision history:
#     11 Apr 2016 J. Sánchez-Gallego
#       Initial version


from __future__ import division
from __future__ import print_function

import numpy as np

from flask_classy import route
from flask import jsonify

from marvin.tools.spaxel import Spaxel
from marvin.api.base import BaseView, arg_validate as av
from marvin.core.exceptions import MarvinError
from marvin.utils.general import parseIdentifier


def _getSpaxel(name, x, y, **kwargs):
    """Retrieves a Marvin Spaxel object."""

    spaxel = None
    results = {}

    # Pop the release to remove a duplicate input to Maps
    release = kwargs.pop('release', None)

    # parse name into either mangaid or plateifu
    try:
        idtype = parseIdentifier(name)
    except Exception as e:
        results['error'] = 'Failed to parse input name {0}: {1}'.format(name, str(e))
        return spaxel, results

    try:
        if idtype == 'plateifu':
            plateifu = name
            mangaid = None
        elif idtype == 'mangaid':
            mangaid = name
            plateifu = None
        else:
            raise MarvinError('invalid plateifu or mangaid: {0}'.format(idtype))

        spaxel = Spaxel(x=x, y=y, mangaid=mangaid, plateifu=plateifu,
                        release=release, **kwargs)
        results['status'] = 1
    except Exception as e:
        results['error'] = 'Failed to retrieve Spaxels {0}: {1}'.format(name, str(e))

    return spaxel, results


class SpaxelView(BaseView):
    """Class describing API calls related to Spaxels."""

    route_base = '/spaxels/'

    @route('/<name>/spectra/<x>/<y>/', methods=['GET', 'POST'], endpoint='getSpectrum')
    @av.check_args()
    def spectrum(self, args, name, x, y):
        """Returns a dictionary with the DRP spectrum for a spaxel.

        Loads a DRP Cube and uses getSpaxel to retrieve the ``(x,y)``
        spaxel. Returns a dictionary with the spectrum for that spaxel.

        .. :quickref: Spaxel; Get a spectrum from a specific spaxel from a DRP cube

        :param name: The name of the object as plate-ifu or mangaid
        :param x: The x coordinate of the spaxel (origin is ``lower``)
        :param y: The y coordinate of the spaxel (origin is ``lower``)
        :form inconfig: json of any incoming parameters
        :resjson int status: status of response. 1 if good, -1 if bad.
        :resjson string error: error message, null if None
        :resjson json inconfig: json of incoming configuration
        :resjson json utahconfig: json of outcoming configuration
        :resjson string traceback: traceback of an error, null if None
        :resjson json data: dictionary of returned data
        :json list flux: the spectrum flux array
        :json list ivar: the spectrum ivar array
        :json list mask: the spectrum mask array
        :json list wavelength: the spectrum wavelength array
        :json list specres: the spectrum spectral resolution array
        :resheader Content-Type: application/json
        :statuscode 200: no error
        :statuscode 422: invalid input parameters

        **Example request**:

        .. sourcecode:: http

           GET /marvin2/api/spaxels/8485-1901/spectra/10/10/ HTTP/1.1
           Host: api.sdss.org
           Accept: application/json, */*

        **Example response**:

        .. sourcecode:: http

           HTTP/1.1 200 OK
           Content-Type: application/json
           {
              "status": 1,
              "error": null,
              "inconfig": {"release": "MPL-5"},
              "utahconfig": {"release": "MPL-5", "mode": "local"},
              "traceback": null,
              "data": {"flux": [-0.001416, 0.0099, 0.0144, ...],
                    "ivar": [134.613, 133.393, 132.094, ...],
                    "mask": [0, 0, 0, ...],
                    "wavelength": [3621.6, 3622.43, ..., 10353.8],
                    "specres": [1026.83, 1027.07, 1027.3]
              }
           }

        """

        # Pop any args we don't want going into Spaxel
        args = self._pop_args(args, arglist=['name', 'x', 'y'])

        spaxel, results = _getSpaxel(name, x, y, maps=False, modelcube=False, **args)

        self.update_results(results)

        if spaxel is not None:
            self.results['data'] = {'flux': spaxel.spectrum.flux.tolist(),
                                    'ivar': spaxel.spectrum.ivar.tolist(),
                                    'mask': spaxel.spectrum.mask.tolist(),
                                    'wavelength': spaxel.spectrum.wavelength.tolist(),
                                    'specres': spaxel.specres.tolist()}

        return jsonify(self.results)

    @route('/<name>/properties/<template_kin>/<x>/<y>/',
           methods=['GET', 'POST'], endpoint='getProperties')
    @av.check_args()
    def properties(self, args, name, x, y, template_kin):
        """Returns a dictionary with the DAP properties for a spaxel.

        Loads a DAP Maps and uses getSpaxel to retrieve the ``(x,y)``
        spaxel. Returns a dictionary with the properties for that spaxel.

        .. :quickref: Spaxel; Get DAP properties from a specific spaxel from a DAP Maps

        :param name: The name of the object as plate-ifu or mangaid
        :param x: The x coordinate of the spaxel (origin is ``lower``)
        :param y: The y coordinate of the spaxel (origin is ``lower``)
        :param template_kin: The template_kin associated with this maps. If none, default is used.
        :form inconfig: json of any incoming parameters
        :resjson int status: status of response. 1 if good, -1 if bad.
        :resjson string error: error message, null if None
        :resjson json inconfig: json of incoming configuration
        :resjson json utahconfig: json of outcoming configuration
        :resjson string traceback: traceback of an error, null if None
        :resjson json data: dictionary of returned data
        :json dict properties: the DAP properties for this spaxel
        :resheader Content-Type: application/json
        :statuscode 200: no error
        :statuscode 422: invalid input parameters

        **Example request**:

        .. sourcecode:: http

           GET /marvin2/api/spaxels/8485-1901/properties/GAU-MILESHC/10/10/ HTTP/1.1
           Host: api.sdss.org
           Accept: application/json, */*

        **Example response**:

        .. sourcecode:: http

           HTTP/1.1 200 OK
           Content-Type: application/json
           {
              "status": 1,
              "error": null,
              "inconfig": {"release": "MPL-5"},
              "utahconfig": {"release": "MPL-5", "mode": "local"},
              "traceback": null,
              "data": {"properties": {
                          "bin_area": {
                            "channel": null,
                            "description": "Area of each bin.",
                            "ivar": null,
                            "mask": null,
                            "name": "bin_area",
                            "unit": "arcsec^2",
                            "value": 0.5
                          },
                          ...
                       }
              }
           }

        """

        # Pop any args we don't want going into Spaxel
        args = self._pop_args(args, arglist=['name', 'x', 'y'])

        spaxel, results = _getSpaxel(name, x, y, cube=False, modelcube=False, **args)

        self.update_results(results)

        if spaxel is not None:
            spaxel_properties = {}
            for name in spaxel.properties:
                prop = spaxel.properties[name]
                spaxel_properties[name] = {}
                for key in ['name', 'channel', 'value', 'ivar', 'mask', 'description', 'unit']:
                    propval = getattr(prop, key)
                    if type(propval).__module__ == np.__name__:
                        propval = np.asscalar(propval)
                    spaxel_properties[name][key] = propval

            self.results['data'] = {'properties': spaxel_properties}

        return jsonify(self.results)

    @route('/<name>/models/<template_kin>/<x>/<y>/',
           methods=['GET', 'POST'], endpoint='getModels')
    @av.check_args()
    def getModels(self, args, name, x, y, template_kin):
        """Returns a dictionary with the models for a spaxel.

        Loads a ModelCube and uses getSpaxel to retrieve the ``(x,y)``
        spaxel. Returns a dictionary with the models for that spaxel.

        .. :quickref: Spaxel; Get the models for a specific spaxel from a DAP ModelCube

        :param name: The name of the object as plate-ifu or mangaid
        :param x: The x coordinate of the spaxel (origin is ``lower``)
        :param y: The y coordinate of the spaxel (origin is ``lower``)
        :param template_kin: The template_kin associated with this maps. If none, default is used.
        :form inconfig: json of any incoming parameters
        :resjson int status: status of response. 1 if good, -1 if bad.
        :resjson string error: error message, null if None
        :resjson json inconfig: json of incoming configuration
        :resjson json utahconfig: json of outcoming configuration
        :resjson string traceback: traceback of an error, null if None
        :resjson json data: dictionary of returned data
        :json list flux_array: flux of the binned spectrum
        :json list flux_ivar: ivar of the binned spectrum
        :json list flux_mask: mask of the binned spectrum and model
        :json list model_array: best fitting model spectra
        :json list model_emline: model spectrum with only emission lines
        :json list model_emline_base: model of constant baseline fitted beneath emission lines
        :json list model_emline_mask: bitmask that applies only to emission-line modeling
        :json string bintype: the spectrum spectral resolution array
        :json string template_kin: the spectrum spectral resolution array
        :resheader Content-Type: application/json
        :statuscode 200: no error
        :statuscode 422: invalid input parameters

        **Example request**:

        .. sourcecode:: http

           GET /marvin2/api/spaxels/8485-1901/models/GAU-MILESHC/10/10/ HTTP/1.1
           Host: api.sdss.org
           Accept: application/json, */*

        **Example response**:

        .. sourcecode:: http

           HTTP/1.1 200 OK
           Content-Type: application/json
           {
              "status": 1,
              "error": null,
              "inconfig": {"release": "MPL-5"},
              "utahconfig": {"release": "MPL-5", "mode": "local"},
              "traceback": null,
              "data": {"bintype": "SPX",
                    "template_kin": "GAU-MILESHC",
                    "flux_array": [-0.001416, 0.0099, 0.0144, ...],
                    "flux_ivar": [134.613, 133.393, 132.094, ...],
                    "flux_mask": [32, 32, 32, ...],
                    "model_array": [0, 0, 0, ...],
                    "model_emline": [0, 0, 0, ...],
                    "model_emline_base": [0, 0, 0, ...],
                    "model_emline_mask": [128, 128, 128, ...],
              }
           }

        """

        # Pop any args we don't want going into Spaxel
        args = self._pop_args(args, arglist=['name', 'x', 'y'])

        spaxel, results = _getSpaxel(name, x, y, cube=False, maps=False, **args)

        self.update_results(results)

        if spaxel is not None:

            self.results['data'] = {
                'flux_array': spaxel.model_flux.flux.tolist(),
                'flux_ivar': spaxel.model_flux.ivar.tolist(),
                'flux_mask': spaxel.model_flux.mask.tolist(),
                'model_array': spaxel.model.flux.tolist(),
                'model_emline': spaxel.emline.flux.tolist(),
                'model_emline_base': spaxel.emline_base.flux.tolist(),
                'model_emline_mask': spaxel.emline.mask.tolist(),
                'bintype': spaxel.bintype,
                'template_kin': spaxel.template_kin}

        return jsonify(self.results)



# !usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Licensed under a 3-clause BSD license.
#
# @Author: Brian Cherinka
# @Date:   2017-06-12 18:20:03
# @Last modified by:   Brian Cherinka
# @Last Modified time: 2017-07-05 14:22:58

from __future__ import print_function, division, absolute_import
from marvin import config
from marvin.tools.cube import Cube
import pytest
import os


class TestMisc(object):

    @pytest.mark.parametrize('mpl, drpver', [('MPL-4', 'v1_5_1')])
    def test_custom_drpall(self, galaxy, mpl, drpver):
        assert galaxy.drpall in config.drpall
        cube = Cube(plateifu=galaxy.plateifu, release=mpl)
        drpall = 'drpall-{0}.fits'.format(drpver)
        assert cube._release == mpl
        assert cube._drpver == drpver
        assert os.path.exists(cube._drpall) is True
        assert drpall in cube._drpall
        assert galaxy.drpall in config.drpall



# Copyright 2015 SolidBuilds.com. All rights reserved.
#
# Authors: Ling Thio <ling.thio@gmail.com>

from __future__ import print_function
from sqlalchemy_boolean_search import parse_boolean_search


def test_boolean_expressions():
    # Test precedence
    expression = parse_boolean_search('a=1 or b=2 or not c=3 and d=4 and e=5')
    assert repr(expression) == 'or_(a=1, b=2, and_(not_(c=3), d=4, e=5))'

    # Test parenthesis
    expression = parse_boolean_search('a=1 or not b=2 and c=3')
    assert repr(expression) == 'or_(a=1, and_(not_(b=2), c=3))'

    expression = parse_boolean_search('a=1 or not (b=2 and c=3)')
    assert repr(expression) == 'or_(a=1, not_(and_(b=2, c=3)))'

    expression = parse_boolean_search('(a=1 or not b=2) and c=3')
    assert repr(expression) == 'and_(or_(a=1, not_(b=2)), c=3)'

    # Test all operators
    expression = parse_boolean_search('a < 1 or a <= 1 or a = 1 or a == 1 or a >= 1 or a > 1 or a != 1')
    assert repr(expression) == 'or_(a<1, a<=1, a=1, a==1, a>=1, a>1, a!=1)'

    # Test example
    expression = parse_boolean_search('field1=*something* and not (field2==1 or field3<=10.0)')
    assert repr(expression) == 'and_(field1=*something*, not_(or_(field2==1, field3<=10.0)))'

    # Test range
    expr = parse_boolean_search('a >= 4 and a < 6')
    assert repr(expr) == 'and_(a>=4, a<6)'

    # Test between
    expr = parse_boolean_search('a between 1 and 2')
    assert repr(expr) == 'abetween1and2'

    expr = parse_boolean_search('a between 1 and 2 or c > 10')
    assert repr(expr) == 'or_(abetween1and2, c>10)'


def test_boolean_params():
    expr = parse_boolean_search('a < 1 and b > 2')
    assert 'a' in expr.params
    assert 'b' in expr.params
    assert sorted(['a', 'b']) == sorted(expr.uniqueparams)
    assert expr.conditions is not []


def test_condition_nobase():
    expr = parse_boolean_search('a < 1 and b > 2')
    cond = expr.conditions[0]
    assert repr(cond) == 'a<1'
    assert cond.name == 'a'
    assert cond.basename is None
    assert cond.fullname == 'a'
    assert cond.op == '<'
    assert cond.value == '1'


def test_condition_base():
    expr = parse_boolean_search('table.a < 1 and b > 2')
    cond = expr.conditions[0]
    assert repr(cond) == 'table.a<1'
    assert cond.name == 'a'
    assert cond.basename == 'table'
    assert cond.fullname == 'table.a'
    assert cond.op == '<'
    assert cond.value == '1'





#!/usr/bin/env python
# encoding: utf-8

'''
Created by Brian Cherinka on 2016-05-02 16:16:27
Licensed under a 3-clause BSD license.

Revision History:
    Initial Version: 2016-05-02 16:16:27 by Brian Cherinka
    Last Modified On: 2016-05-02 16:16:27 by Brian

'''
from __future__ import print_function
from __future__ import division
from flask import session as current_session, render_template, request, g, jsonify
from marvin import config
from marvin.utils.db import get_traceback
from collections import defaultdict
import re
import traceback


def configFeatures(app, mode):
    ''' Configure Flask Feature Flags '''

    app.config['FEATURE_FLAGS']['collab'] = False if mode == 'dr13' else True
    app.config['FEATURE_FLAGS']['new'] = False if mode == 'dr13' else True
    app.config['FEATURE_FLAGS']['dev'] = True if config.db == 'local' else False


def updateGlobalSession():
    ''' updates the Marvin config with the global Flask session '''

    # check if mpl versions in session
    if 'versions' not in current_session:
        setGlobalSession()
    elif 'drpver' not in current_session or \
         'dapver' not in current_session:
        drpver, dapver = config.lookUpVersions(release=config.release)
        current_session['drpver'] = drpver
        current_session['dapver'] = dapver
    elif 'release' not in current_session:
        current_session['release'] = config.release


def setGlobalSession():
    ''' Sets the global session for Flask '''

    mpls = list(config._mpldict.keys())
    versions = [{'name': mpl, 'subtext': str(config.lookUpVersions(release=mpl))} for mpl in mpls]
    current_session['versions'] = versions

    if 'release' not in current_session:
        current_session['release'] = config.release
        drpver, dapver = config.lookUpVersions(release=config.release)
        current_session['drpver'] = drpver
        current_session['dapver'] = dapver


def parseSession():
    ''' parse the current session for MPL versions '''
    drpver = current_session['drpver']
    dapver = current_session['dapver']
    release = current_session['release']
    return drpver, dapver, release


# def make_error_json(error, name, code):
#     ''' creates the error json dictionary for API errors '''
#     shortname = name.lower().replace(' ', '_')
#     messages = {'error': shortname,
#                 'message': error.description if hasattr(error, 'description') else None,
#                 'status_code': code,
#                 'traceback': get_traceback(asstring=True)}
#     return jsonify({'api_error': messages}), code


# def make_error_page(app, name, code, sentry=None, data=None):
#     ''' creates the error page dictionary for web errors '''
#     shortname = name.lower().replace(' ', '_')
#     error = {}
#     error['title'] = 'Marvin | {0}'.format(name)
#     error['page'] = request.url
#     error['event_id'] = g.get('sentry_event_id', None)
#     error['data'] = data
#     if sentry:
#         error['public_dsn'] = sentry.client.get_public_dsn('https')
#     app.logger.error('{0} Exception {1}'.format(name, error))
#     return render_template('errors/{0}.html'.format(shortname), **error), code


def send_request():
    ''' sends the request info to the history db '''
    if request.blueprint is not None:
        print(request.cookies)
        print(request.headers)
        print(request.blueprint)
        print(request.endpoint)
        print(request.url)
        print(request.remote_addr)
        print(request.environ)
        print(request.environ['REMOTE_ADDR'])

# def setGlobalSession_old():
#     ''' Set default global session variables '''

#     if 'currentver' not in current_session:
#         setGlobalVersion()
#     if 'searchmode' not in current_session:
#         current_session['searchmode'] = 'plateid'
#     if 'marvinmode' not in current_session:
#         current_session['marvinmode'] = 'mangawork'
#     configFeatures(current_app, current_session['marvinmode'])
#     # current_session['searchoptions'] = getDblist(current_session['searchmode'])

#     # get code versions
#     #if 'codeversions' not in current_session:
#     #    buildCodeVersions()

#     # user authentication
#     if 'http_authorization' not in current_session:
#         try:
#             current_session['http_authorization'] = request.environ['HTTP_AUTHORIZATION']
#         except:
#             pass


# def getDRPVersion():
#     ''' Get DRP version to use during MaNGA SAS '''

#     # DRP versions
#     vers = marvindb.session.query(marvindb.datadb.PipelineVersion).\
#         filter(marvindb.datadb.PipelineVersion.version.like('%v%')).\
#         order_by(marvindb.datadb.PipelineVersion.version.desc()).all()
#     versions = [v.version for v in vers]

#     return versions


# def getDAPVersion():
#     ''' Get DAP version to use during MaNGA SAS '''

#     # DAP versions
#     vers = marvindb.session.query(marvindb.datadb.PipelineVersion).\
#         join(marvindb.datadb.PipelineInfo, marvindb.datadb.PipelineName).\
#         filter(marvindb.datadb.PipelineName.label == 'DAP',
#                ~marvindb.datadb.PipelineVersion.version.like('%trunk%')).\
#         order_by(marvindb.datadb.PipelineVersion.version.desc()).all()
#     versions = [v.version for v in vers]

#     return versions+['NA']


# def setGlobalVersion():
#     ''' set the global version '''

#     # set MPL version
#     try:
#         mplver = current_session['currentmpl']
#     except:
#         mplver = None
#     if not mplver:
#         current_session['currentmpl'] = 'MPL-4'

#     # set version mode
#     try:
#         vermode = current_session['vermode']
#     except:
#         vermode = None
#     if not vermode:
#         current_session['vermode'] = 'MPL'

#     # initialize
#     if 'MPL' in current_session['vermode']:
#         setMPLVersion(current_session['currentmpl'])

#     # set global DRP version
#     try:
#         versions = current_session['versions']
#     except:
#         versions = getDRPVersion()
#     current_session['versions'] = versions
#     try:
#         drpver = current_session['currentver']
#     except:
#         drpver = None
#     if not drpver:
#         realvers = [ver for ver in versions if os.path.isdir(os.path.join(os.getenv('MANGA_SPECTRO_REDUX'), ver))]
#         current_session['currentver'] = realvers[0]

#     # set global DAP version
#     try:
#         dapversions = current_session['dapversions']
#     except:
#         dapversions = getDAPVersion()
#     current_session['dapversions'] = dapversions
#     try:
#         ver = current_session['currentdapver']
#     except:
#         ver = None
#     if not ver:
#         realvers = [ver for ver in versions if os.path.isdir(os.path.join(os.getenv('MANGA_SPECTRO_ANALYSIS'),
#                     current_session['currentver'], ver))]
#         current_session['currentdapver'] = realvers[0] if realvers else 'NA'


def buildImageDict(imagelist, test=None, num=16):
    ''' Builds a list of dictionaries from a sdss_access return list of images '''

    # get thumbnails and plateifus
    if imagelist:
        thumbs = [imagelist.pop(imagelist.index(t)) if 'thumb' in t else t for t in imagelist]
        plateifu = ['-'.join(re.findall('\d{3,5}', im)) for im in imagelist]

    # build list of dictionaries
    images = []
    if imagelist:
        for i, image in enumerate(imagelist):
            imdict = defaultdict(str)
            imdict['name'] = plateifu[i]
            imdict['image'] = image
            imdict['thumb'] = thumbs[i] if thumbs else None
            images.append(imdict)
    elif test and not imagelist:
        for i in xrange(num):
            imdict = defaultdict(str)
            imdict['name'] = '4444-0000'
            imdict['image'] = 'http://placehold.it/470x480&text={0}'.format(i)
            imdict['thumb'] = 'http://placehold.it/150x150&text={0}'.format(i)
            images.append(imdict)

    return images
